#!/bin/bash

if [ "${DXC_HOME:-x}" = x ]; then
  echo Setting DXC_HOME to `pwd`
  export DXC_HOME=`pwd`
fi

export LD_LIBRARY_PATH=$DXC_HOME/Lib
export JAVA_TESTS_VAL=false

if [ "${JAVA_HOME:-x}" = x ]; then
  if [ "$1" \!= "nojava" ]; then
    echo Setting JAVA_HOME to $1
    export JAVA_HOME=$1
    export JAVA_TESTS_VAL=true
  fi
else
  export JAVA_TESTS_VAL=true
fi

cmake -DRUN_DOXYGEN=true -DBUILD_CPP_TESTS=true -DBUILD_JAVA_TESTS=$JAVA_TESTS_VAL .

if [ $? -ne 0 ]; then
  exit 1;
fi

make install

if [ $? -ne 0 ]; then
  exit 1;
fi

if [ "$1" \!= "nojava" ]; then
  cd Src/
  ant install

  if [ $? -ne 0 ]; then
    echo command \`ant install\':
    echo buildAndTest:0: warning: Java build failed
    if [ "${JAVA_HOME:+x}" = x ]; then
      echo Make sure JAVA_HOME points to a working JSDK
      exit 1;
    fi
  fi

  cd ..
fi

echo
echo Running Cpp tests
$DXC_HOME/Bin/TestRunner

if [ $? -ne 0 ]; then
  exit 1;
fi

if [ "$1" \!= "nojava" ]; then
  echo
  echo Running Java tests
  java -cp "$DXC_HOME/Lib/DxcApi.jar:$DXC_HOME/Bin/Evaluator.jar:$DXC_HOME/Src/JavaTests/junit-4.6.jar:$DXC_HOME/Bin/JavaTests.jar" org.junit.runner.JUnitCore org.dxc.tests.TestParser org.dxc.tests.TestDAResult

  if [ $? -ne 0 ]; then
    exit 1;
  fi
fi

