/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef COMMAND_HPP
#define COMMAND_HPP

#include <iostream>
#include <string>
#include <set>

#include <DxcDefs.hpp>
#include <DataTypes/Value.hpp>

namespace Dxc
{

/**
 * @brief Representation of a component command, with parameters.
 */
class DXCAPI_EXPORT Command
{
public:
    Command(const std::string& commandID, const Value* value);

    std::string getCommandID() const;
    const Value* getValue() const;

    /**
     * @brief Print the value to an output stream.
     */
    std::ostream &put(std::ostream &) const;
    bool operator==(const Dxc::Command& f1) const;
    bool operator<(const Dxc::Command& f1) const;


    Command(const Command&);
    Command& operator=(const Command&);
    ~Command();

private:
    std::string commandID_;
    Value* value_;
};

typedef DXCAPI_EXPORT std::set<Command> cmdSet;

/**
 * @brief A set of Commands.
 */
class DXCAPI_EXPORT CommandSet : public cmdSet
{
public:
//    bool operator<(const CommandSet&) const;
//    bool operator==(const CommandSet&) const;
    std::ostream &put(std::ostream &) const;
};

}

std::ostream DXCAPI_EXPORT & operator << (std::ostream &, const Dxc::CommandSet&);

std::ostream DXCAPI_EXPORT & operator << (std::ostream &, const Dxc::Command&);

#endif
