/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#ifndef SENSOR_DATA_HPP
#define SENSOR_DATA_HPP

#include <map>
#include <DxcDefs.hpp>
#include <DxcDataTypes.hpp>

namespace Dxc
{
/**
 * @brief Data structure for sensor data messages.
 */
class DXCAPI_EXPORT SensorData : public DxcData
{
public:
    /**
     * @brief Typedef for mapping sensor name to Value.
     */
    typedef std::map<std::string, const Value*> SensorValueMap;

    /**
     * @brief Constructor.
     *
     * @param timestamp - message timestamp.
     * @param sensorMap - map containing data points for each sensor.
     */
    SensorData(Timestamp timestamp, const SensorValueMap& sensorMap);

    virtual std::ostream &put(std::ostream &) const;

    /**
     * @brief Returns the map from sensor to Value.
     * Deprecated, use getSensors() instead.
     *
     * @deprecated Use getSensors() instead.
     *
     * If the type of datapoint is unknown (it should be made known, for
     * example through an XML system catalog), the Value type for each
     * point must be determined by trial and error (using RTTI or
     * dynamic_cast null tests).
     */
    DEPRECATED(SensorValueMap getSensorValueMap() const);

    /**
     * @brief Returns the map from sensor to Value (pointer).
     */
    const SensorValueMap* getSensors() const;

    virtual SensorData* clone() const;
    SensorData(const SensorData&);
    SensorData& operator=(const SensorData&);
    ~SensorData();
private:
    SensorData();

    SensorValueMap sensorMap_;
};
}

#endif
