/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include "Value.hpp"
#include <cmath>
#include <typeinfo>
#include <sstream>

using namespace std;
using namespace Dxc;

Value::Value()
{
}

Value* Value::v(const string& s)
{
    return new StringValue(s);
}
Value* Value::v(const char* s)
{
    return new StringValue(s);
}
Value* Value::v(int n)
{
    return new IntegerValue(n);
}
Value* Value::v(double d)
{
    return new RealValue(d);
}
Value* Value::v(bool b)
{
    return new BoolValue(b);
}

IntegerValue::IntegerValue(int value) : value_(value)
{
}

ostream &IntegerValue::put(ostream &out) const
{
    out << value_;

    return out;
}

int IntegerValue::get() const
{
    return value_;
}

Value* IntegerValue::clone() const
{
    return new IntegerValue(*this);
}

BoolValue::BoolValue(bool value) : value_(value)
{
}

ostream &BoolValue::put(ostream &out) const
{
    out << (value_ ? "true" : "false");

    return out;
}

bool BoolValue::get() const
{
    return value_;
}

Value* BoolValue::clone() const
{
    return new BoolValue(*this);
}

RealValue::RealValue(double value) : value_(value)
{
}

ostream &RealValue::put(ostream &out) const
{
    out << value_;
    long double d;

    ostringstream iss;
    string::size_type i;
    iss << value_;
    i = iss.str().find('e');
    if (i == string::npos)
        i = iss.str().find('E');

    if (i == string::npos && modf(value_, &d) == 0.0)
        out << ".0";

    return out;
}

double RealValue::get() const
{
    return value_;
}

Value* RealValue::clone() const
{
    return new RealValue(*this);
}

StringValue::StringValue(const string *value) : value_(*value)
{
}

StringValue::StringValue(const string &value) : value_(value)
{
}

StringValue::StringValue(const char *value) : value_(value)
{
}

ostream &StringValue::put(ostream &out) const
{
    out << "'" << value_ << "'";

    return out;
}

string StringValue::get() const
{
    return value_;
}

Value* StringValue::clone() const
{
    return new StringValue(*this);
}

ostream & operator << (ostream &out, const Dxc::Value *value)
{
    return value->put(out);
}

bool Value::lt(const Dxc::Value *v) const
{
    ostringstream oss1, oss2;
    oss1 << this;
    oss2 << v;
    return oss1.str() < oss2.str();
}

bool Value::eq(const Dxc::Value *v) const
{
    if (string(typeid(*this).name()) != string(typeid(*v).name()))
        return false;

    if (string(typeid(*v).name()) ==
            string(typeid(StringValue).name()))
    {
        return dynamic_cast<const StringValue*>(this)->get() ==
               dynamic_cast<const StringValue*>(v)->get();
    }
    else if (string(typeid(*v).name()) ==
             string(typeid(BoolValue).name()))
    {
        return dynamic_cast<const BoolValue*>(this)->get() ==
               dynamic_cast<const BoolValue*>(v)->get();
    }
    else if (string(typeid(*v).name()) ==
             string(typeid(IntegerValue).name()))
    {
        return dynamic_cast<const IntegerValue*>(this)->get() ==
               dynamic_cast<const IntegerValue*>(v)->get();
    }
    else if (string(typeid(*v).name()) == string(typeid(RealValue).name()))
    {
        return dynamic_cast<const RealValue*>(this)->get() ==
               dynamic_cast<const RealValue*>(v)->get();
    }

    return false;
}
