/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.api.connection;

import java.lang.Runnable;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import org.dxc.api.DxcCallback;
import org.dxc.api.DxcDataGenerator;
import org.dxc.api.parser.DxcParser;

public class StreamListener implements Runnable {
    private DxcCallback callback;
    private String source = null;
    private Reader stream;

    StreamListener(DxcCallback c, InputStream s) {
        this(c, new InputStreamReader(s));
    }

    StreamListener(DxcCallback c, Reader s) {
        this.callback = c;
        this.stream = s;
    }

    StreamListener(DxcCallback c, Reader s, String source) {
        this.callback = c;
        this.stream = s;
        this.source = source;
    }

    public StreamListener(DxcCallback c, InputStream s, String source) {
        this(c, new InputStreamReader(s), source);
    }

    public void run() {
        DxcDataGenerator dgen = new DxcParser(stream);
        try {
            if (source != null)
                dgen.begin(callback, source);
            else
                dgen.begin(callback);
            stream.close();
        } catch (Exception ex) {
            System.out.print(ex);
        }
    }
}