/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.api.datatypes;

import java.io.OutputStream;
import java.io.PrintStream;

/**
 * Error message.
 */
public class ErrorData extends DxcData {

    private String error;

    /**
     * Constructor.
     * 
     * @param timestamp
     *            - message time.
     * @param error
     *            - string describing the error.
     */
    public ErrorData(long timestamp, String error) {
        super(timestamp);
        if (error == null)
            this.error = "";
        else
            this.error = error.replaceAll("[^\\\\]'", "\\\\'");
    }

    /**
     * Constructor.
     * 
     * @param error
     *            - string describing the error.
     */
    public ErrorData(String error) {
        super(DxcData.getCurrentTime());
        if (error == null)
            this.error = "";
        else
            this.error = error.replaceAll("[^\\\\]'", "\\\\'");
    }

    /**
     * Returns the error message string.
     */
    public String getError() {
        return error;
    }

    public OutputStream put(OutputStream os) {
        PrintStream out = new PrintStream(os);
        out.append("error @");
        out.append(Long.toString(this.getTimeStamp()));
        if (this.getSource() != null)
            out.append(" <" + this.getSource() + ">");
        out.append(" '" + error + "';\n");
        return out;
    }

    public boolean equals(Object o) {
        if (o == this)
            return true;
        if ((o == null) || (o.getClass() != this.getClass()))
            return false;

        ErrorData cd = (ErrorData) o;

        if ((cd.getTimeStamp() != getTimeStamp())
                || (!cd.getSource().equals(getSource()))
                || (!cd.getError().equals(getError())))
            return false;

        return true;
    }

    public int hashCode() {
        int hash = (int) getTimeStamp();
        hash = 31 * hash + getSource().hashCode();
        hash = 31 * hash + error.hashCode();
        return hash;
    }
}