/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia
  
  The DXC Framework is free software: you can redistribute it and/or modify it 
  under the terms of the GNU Lesser General Public License as published by the 
  Free Software Foundation, either version 3 of the License, or (at your 
  option) any later version. 

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
  details. 

  You should have received a copy of the GNU Lesser General Public License 
  along with this program. If not, see <http://www.gnu.org/licenses/> 
 */
package org.dxc.api.datatypes;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;

/**
 * Abstract class for values that can be of various concrete datatypes.
 * 
 * Value is the superclass of all the basic datatypes that can be referred to by
 * DxcData, such as boolean, integer, real, or string. The abstraction is
 * provided so that data of various kinds can be supplied in a uniform way, eg.
 * for sensor values and command values.
 */
public abstract class Value {
    /**
     * Print the value to an output stream.
     */
    public abstract OutputStream put(OutputStream out);

    public String toString() {
        OutputStream bos = new ByteArrayOutputStream();
        put(bos);
        return bos.toString();
    }

    public static Value v(String s) {
        return new StringValue(s);
    }

    public static Value v(int n) {
        return new IntegerValue(n);
    }

    public static Value v(double d) {
        return new RealValue(d);
    }

    public static Value v(boolean b) {
        return new BoolValue(b);
    }
}