package org.dxc.evaluator.metrics;

import static org.junit.Assert.*;

import org.junit.Test;
import org.dxc.api.datatypes.*;

public class TestIsolationTime {
    @Test
    public void defaultNaN() {
        IsolationTime il = new IsolationTime();

        assertEquals("Score should be NaN by default", Double.NaN, il
                .getScore(), 0.0001);
    }

    @Test
    public void isolationTime() {
        CandidateSet cs = new CandidateSet();
        Candidate c = new Candidate();
        IsolationTime iso = new IsolationTime();

        c.getFaultSet().add(new Fault("a", "b"));
        cs.add(c);

        iso.addData(new FaultInjectData(0, new Fault("c", "d"), true));
        for (int n = 0; n < 3; n++) {
            iso.addData(new DiagnosisData(7 + n * 500, true, true, cs, ""));
        }
        iso.calculate();

        assertEquals(7, iso.getScore(), 0);
    }

    @Test
    public void prevIsolation() {
        CandidateSet cs = new CandidateSet();
        Candidate c = new Candidate();

        c.getFaultSet().add(new Fault("a", "b"));
        cs.add(c);

        IsolationTime iso = new IsolationTime(new DiagnosisData(0, true, true,
                cs, ""));
        iso.addData(new FaultInjectData(1, new Fault("c", "d"), true));

        c.getFaultSet().add(new Fault("b", "c"));
        for (int n = 0; n < 3; n++) {
            iso.addData(new DiagnosisData(7 + n * 500, true, true, cs, ""));
        }
        iso.calculate();

        assertEquals(Double.NaN, iso.getScore(), 0);
    }
}
