/*
  DXC Framework (http://dx-competition.org/)
  Copyright 2008 David Garcia

  The DXC Framework is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation, either version 3 of the License, or (at your
  option) any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
  details.

  You should have received a copy of the GNU Lesser General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>
*/
#include <vector>
#include <exception>
#include <DxcConfig.hpp>
#include <DataTypes/DxcException.hpp>
#include <JavaLoader.hpp>

using namespace std;
using namespace Dxc;

#ifdef _WIN32
#define CP_DELIM ";"
#define CP_QUOTE "\""
void escapeSpaces(std::string& s)
{
}
#else
#define CP_DELIM ":"
#define CP_QUOTE ""
void escapeSpaces(string& s)
{
    string::size_type n = 0;

    while (n != string::npos)
    {
        n = s.find_first_of(' ', n);
        if (n != string::npos)
        {
            if (n != 0)
            {
                s.insert(n, "\\");
                n++;
            }
            n++;
        }
    }
}
#endif

JavaLoader::JavaLoader(const std::string& jarName,
                       const std::string& className,
                       const std::string& path,
                       const std::string& args)
    : className_(className), args_(args), javavm_(0)
{
    string tmpPath(path);
    string::size_type n = tmpPath.find_first_of('\\');

    string javaApi = DxcConfig::getInstance()->get("JAVA_API");

    while (n != string::npos)
    {
        tmpPath.replace(n, 1, "/");
        n = tmpPath.find_first_of('\\', n + 1);
    }

    this->classpath_ = CP_QUOTE + tmpPath + jarName + CP_QUOTE + CP_DELIM;
    this->classpath_ += CP_QUOTE + javaApi + CP_QUOTE;

    escapeSpaces(this->classpath_);
}

void JavaLoader::start()
{
    string lArgs("-cp " + classpath_ + " " +
                 className_ + " " + args_);

    string javaVM = DxcConfig::getInstance()->get("JAVA_HOME");
    if (javaVM == "")
        throw DxcException("Java VM not found. Check Dxc.cfg and JAVA_HOME environment variable.");

    javaVM.append(1, DIR_DELIM);
    javaVM += "bin";
    javaVM.append(1, DIR_DELIM);

    javavm_ = ProcessFactory::createProcessLoader(
                  "java",
                  javaVM,
                  lArgs);

    if (javavm_)
    {
        javavm_->start();
    }
}

bool JavaLoader::isRunning()
{
    if (javavm_)
    {
        javavm_->isRunning();
    }

    return false;
}

bool JavaLoader::terminate()
{
    if (javavm_)
    {
        return javavm_->terminate();
    }

    return false;
}

bool JavaLoader::timedWait(long time)
{
    return javavm_->timedWait(time);
}

JavaLoader::~JavaLoader()
{
    if (javavm_)
        delete javavm_;
}

unsigned long JavaLoader::getCpuTime()
{
    return javavm_->getCpuTime();
}

unsigned long JavaLoader::getPeakMemoryUsage()
{
    return javavm_->getPeakMemoryUsage();
}
